<?php

namespace Drupal\commerce_qazaqpay\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Drupal\commerce_order\Entity\Order;

/**
 * Provides the Off-site Redirect payment Qazaqpay gateway.
 *
 * @CommercePaymentGateway(
 *   id = "qazaqpay_offsite_gateway",
 *   label = "Qazaqpay Payment Gateway",
 *   display_label = "Qazaqpay",
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_qazaqpay\PluginForm\OffsiteRedirect\QazaqpayForm"
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "mastercard", "visa",
 *   },
 *   modes = {
 *     "live" = @Translation("Live")
 *   }
 * )
 */
class Qazaqpay extends OffsitePaymentGatewayBase
{
    use StringTranslationTrait;

    const STATUS_PAYMENT_AUTHORIZED = 'authorization';
    const STATUS_ORDER_COMPLETE = 'completed';
    const STATUS_PAYMENT_FAIL = 'authorization_voided';
    const STATUS_PAYMENT_REFUNDED = 'refunded';
    const STATUS_PAYMENT_PENDING = 'pending';
    const STATUS_PAYMENT_CANCELED = 'canceled';

    /**
     * {@inheritdoc}
     */
    public function defaultConfiguration()
    {
        $config = \Drupal::config('commerce_qazaqpay.settings');

        return [
            'gateway_address'  => '',
            'gateway_login' => '',
            'gateway_password' => '',
            'agent_id' => '',
            'virtual_pos_id'      => '',
            'payment_type_id'      => '',
        ] + parent::defaultConfiguration();
    }

    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state)
    {
        $form = parent::buildConfigurationForm($form, $form_state);

        $form['gateway_address'] = [
            '#title'         => $this->t('Gateway Address'),
            '#default_value' => $this->configuration['gateway_address'],
            '#type'          => 'textfield',
            '#required'      => TRUE
        ];

        $form['gateway_login'] = [
            '#title'         => $this->t('Gateway Login'),
            '#default_value' => $this->configuration['gateway_login'],
            '#type'          => 'textfield',
            '#required'      => TRUE
        ];

        $form['gateway_password'] = [
            '#title'         => $this->t('Gateway Password'),
            '#default_value' => $this->configuration['gateway_password'],
            '#type'          => 'textfield',
            '#required'      => TRUE
        ];

        $form['agent_id'] = [
            '#title'         => $this->t('Merchant ID'),
            '#description'   => $this->t('Identifier of the Merchant registered in the Payment System IS'),
            '#default_value' => $this->configuration['agent_id'],
            '#type'          => 'number',
            '#required'      => TRUE
        ];

        $form['virtual_pos_id'] = [
            '#title'         => $this->t('Virtual cash register ID'),
            '#default_value' => $this->configuration['virtual_pos_id'],
            '#type'          => 'number',
            '#required'      => TRUE
        ];

        $form['payment_type_id'] = [
            '#title'         => $this->t('Payment type ID'),
            '#description'   => $this->t('Payment type ID, registered in the Payment System IS'),
            '#default_value' => $this->configuration['payment_type_id'],
            '#type'          => 'number',
            '#required'      => TRUE
        ];

        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function validateConfigurationForm(array &$form, FormStateInterface $form_state)
    {
        parent::validateConfigurationForm($form, $form_state);

        $values = $form_state->getValue($form['#parents']);
        $required_fields = [
            'gateway_address',
            'gateway_login',
            'gateway_password',
            'agent_id',
            'virtual_pos_id',
            'payment_type_id',
        ];
        foreach ($required_fields as $key) {
            if (empty($values[$key])) {
                $this->messenger->addError($this->t('Qazaqpay service is not configured for use. Please contact an administrator to resolve this issue'));
                return FALSE;
            }
        }
    }

    /**
     * {@inheritdoc}
     */
    public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
    {
        parent::submitConfigurationForm($form, $form_state);

        if (!$form_state->getErrors()) {
            $values                                  = $form_state->getValue($form['#parents']);
            $this->configuration['gateway_address']          = $values['gateway_address'];
            $this->configuration['gateway_login']         = $values['gateway_login'];
            $this->configuration['gateway_password']           = $values['gateway_password'];
            $this->configuration['agent_id']  = $values['agent_id'];
            $this->configuration['virtual_pos_id']      = $values['virtual_pos_id'];
            $this->configuration['payment_type_id']          = $values['payment_type_id'];
        }
    }

    /**
     * IPN callback.
     *
     * IPN will be called after a succesful payment or changed state
     *
     * @param \Symfony\Component\HttpFoundation\Request $request
     *   The request.
     *
     * @return \Symfony\Component\HttpFoundation\Response|null
     *   The response, or NULL to return an empty HTTP 200 response
     */
    public function onNotify(Request $request)
    {
		$input = (string) file_get_contents("php://input");
		$data = json_decode($input, true);

		if (!isset($data["billId"])) {
			return new Response('', 200);
		}
        $order_id = $data["billId"];
        $order = Order::load($order_id);
        if (!$order) return new Response('', 200);

        $payment_storage = \Drupal::service('entity_type.manager')
            ->getStorage('commerce_payment');

        $last = $payment_storage->loadByProperties([
            'payment_gateway' => $this->parentEntity->id(),
            'order_id' => $order_id,
        ]);
        if (!empty($last)) {
            $payment_storage->delete($last);
        }

        $payment         = $payment_storage->create([
            'state'           => self::STATUS_ORDER_COMPLETE,
            'amount'          => $order->getTotalPrice(),
            'payment_gateway' => $this->parentEntity->id(),
            'order_id'        => $order_id,
            'test'            => false,
        ]);
        $payment->save();

        $order->set('state', self::STATUS_ORDER_COMPLETE);
        $order->save();

        return new Response('Received new payment status', 200);
    }
}
