<?php

namespace Drupal\commerce_qazaqpay\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_order\Entity\OrderInterface;

/**
 * Provides the class for payment off-site form.
 *
 * Provide a buildConfigurationForm() method which calls buildRedirectForm()
 * with the right parameters.
 */
class QazaqpayForm extends BasePaymentOffsiteForm
{
    /**
     * Gateway plugin.
     *
     * @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface
     */
    private $paymentGatewayPlugin;

    /**
     * Getting plugin's configuration.
     *
     * @param string $configuration
     *   Configuration name.
     *
     * @return mixed
     *   Configuration value.
     */
    private function getConfiguration($configuration)
    {
        return $this->paymentGatewayPlugin->getConfiguration()[$configuration];
    }


    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state)
    {
        $form = parent::buildConfigurationForm($form, $form_state);

        /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
        $payment = $this->entity;
        $this->paymentGatewayPlugin = $payment->getPaymentGateway()->getPlugin();

        $order        = $payment->getOrder();
        $token = $this->getGatewayToken();
        $redirect_url = $this->initPayment($order, $token, $form['#return_url']);

        return $this->buildRedirectForm($form, $form_state, $redirect_url, array(), 'get');
    }

    /**
     * Get gateway auth token for building redirect url.
     *
     * @throws \Drupal\commerce_payment\Exception\PaymentGatewayException
     *
     * @return string
     */
    public function getGatewayToken()
    {
        $body = json_encode(array(
            'login' => $this->getConfiguration('gateway_login'),
            'password' => $this->getConfiguration('gateway_password')
        ), true);

        $ctp_url = $this->getConfiguration('gateway_address') . '/api/auth/login';

        $curl = curl_init($ctp_url);
        curl_setopt($curl, CURLOPT_HEADER, 0);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
        curl_setopt($curl, CURLOPT_FORBID_REUSE, 1);
        curl_setopt($curl, CURLOPT_FRESH_CONNECT, 1);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $body);

        $response = curl_exec($curl);
        $curl_error = curl_error($curl);
        $curl_errno = curl_errno($curl);
        curl_close($curl);

        if (!$response) {
            throw new PaymentGatewayException("Qazaqpay token request failed: $curl_error ($curl_errno)");
        }

        $data = json_decode($response, true);

        if (!isset($data["token"])) {
            throw new PaymentGatewayException("Qazaqpay token request failed: $data");
        }

        return $data["token"];
    }

    /**
     * Get the billing address for this order.
     *
     * @param Order $order
     *   The commerce order object.
     *
     * @throws \Drupal\commerce_payment\Exception\PaymentGatewayException
     *
     * @return string
     */
    public function initPayment(OrderInterface $order, $token, $success_url)
    {
        $billingAddress = $order->getBillingProfile()->get('address')->first();
        $host = \Drupal::request()->getSchemeAndHttpHost();

        $body = array(
            "virtual_pos_id" => intval($this->getConfiguration('virtual_pos_id')),
            "agent_id" => intval($this->getConfiguration('agent_id')),
            "success_url" => $success_url,
            "error_url" => $success_url,
            "notify_url" => $success_url,
            "webhook_url" => $host . "/payment/notify/qazaqpay",
            "client_info" => array(
                "full_name" => $billingAddress->getGivenName() . ' ' . $billingAddress->getFamilyName(),
            ),
            "bill" => array(
                "bill_id" => intval($order->id()),
                "sum" => floatval($order->getTotalPrice()->getNumber()),
                "currency" => "KZT",
                "claims" => array(
                    array(
                        "claim_id" => intval($order->id()),
                        "sum" => floatval($order->getTotalPrice()->getNumber()),
                        "payment_type_id" => intval($this->getConfiguration('payment_type_id')),
                    )
                )
            )
        );

        $ctp_url = $this->getConfiguration('gateway_address') . '/api/banks/call_payment_page';

        $curl = curl_init($ctp_url);
        curl_setopt($curl, CURLOPT_HEADER, 0);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json',
            'Authorization: Bearer ' . $token
        ));
        curl_setopt($curl, CURLOPT_FORBID_REUSE, 1);
        curl_setopt($curl, CURLOPT_FRESH_CONNECT, 1);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($body, true));

        $response = curl_exec($curl);
        $curl_error = curl_error($curl);
        $curl_errno = curl_errno($curl);
        curl_close($curl);

        if (!$response) {
            throw new PaymentGatewayException("Qazaqpay token request failed: $curl_error ($curl_errno)");
        }

        $data = json_decode($response, true);

        if (!isset($data["url"])) {
            throw new PaymentGatewayException("Qazaqpay token request failed: $data");
        }

        return $data["url"];
    }
}
